

#include "c8051.h"
#include "_typedef.h"
#include "waitloop.h"

#define PS2CLK P0_6
#define PS2DAT P0_7





static int countbits(int c)
{
	int n;
	for (n = 0; c; c >>= 1)
		if (c & 1) n++;
	return n;
}



bool pc_get_clock()
{
	return PS2CLK ? true : false;
}

void pc_set_clock_0()
{
	PS2CLK = 0;
	P0MDOUT |= 0x40;
}

void pc_set_clock_1()
{
	P0MDOUT &= ~0x40;
	PS2CLK = 1;
}

bool pc_get_data()
{
	return PS2DAT ? true : false;
}

void pc_set_data_0()
{
	PS2DAT = 0;
	P0MDOUT |= 0x80;
}

void pc_set_data_1()
{
	P0MDOUT &= ~0x80;
	PS2DAT = 1;
}



bool pc_send(int c)
{
	unsigned char i;
	unsigned short bits;

	bits = c & 0xff;
	if (!(countbits(bits) & 1)) bits |= 0x100;
	bits = (bits | 0x200) << 1;
	for (i = 0; i < 11; i++) {
		if (!pc_get_clock()) break;
		if (bits & 1)	pc_set_data_1();
		else			pc_set_data_0();
		bits >>= 1;
		wait_15us();
		if (!pc_get_clock()) break;
		pc_set_clock_0();
		wait_40us();
		pc_set_clock_1();
		wait_15us();
		wait_15us();
	}
	pc_set_data_1();

	return i == 11;
}

int pc_recv()
{
	int i;
	unsigned short bits;

	if (pc_get_data()) return -1;
	if (!pc_get_clock()) return -1;

	bits = 0;
	for (i = 0; i < 10; i++) {
		pc_set_clock_0();
		wait_40us();
		pc_set_clock_1();
		wait_15us();
		bits >>= 1;
		if (pc_get_data()) bits |= 0x200;
	}
	wait_40us();
	pc_set_data_0();
	wait_15us();
	pc_set_clock_0();
	wait_15us();
	pc_set_clock_1();
	pc_set_data_1();

	if (!(bits & 0x200))
		return -1;							// framing error

	if (!(countbits(bits & 0x1ff) & 1))
		return -1;							// parity error

	return bits & 0xff;
}


